import argparse
import inspect
from overrides import overrides
from typing import List

from repro.commands.subcommand import RootSubcommand
from repro.common.logging import prepare_global_logging
from repro.common.util import load_dataset_reader, load_model, load_output_writer
from repro.data.dataset_readers import HuggingfaceDatasetsDatasetReader
from repro.data.types import InstanceDict
from repro.models import Model


def predict_with_model(model: Model, instances: List[InstanceDict]) -> List:
    # Find the required arguments for the model's `predict` function, then
    # select all of those fields from the `instances` and pass them through
    # the `predict_batch` function
    args = inspect.getfullargspec(model.predict)
    required_args = args.args
    required_args.remove("self")

    inputs = []
    for instance in instances:
        inputs.append({arg: instance[arg] for arg in required_args})

    predictions = model.predict_batch(inputs)
    if len(predictions) != len(instances):
        raise Exception(
            f"Model returned {len(predictions)} predictions for {len(instances)} instances"
        )
    return predictions


@RootSubcommand.register("predict")
class PredictSubcommand(RootSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = "Predict using a model"
        self.parser = parser.add_parser(
            "predict", description=description, help=description
        )
        self.parser.add_argument(
            "--model-name", required=True, help="The name of the model to predict with"
        )
        self.parser.add_argument(
            "--model-args",
            required=False,
            help="A serialized json object which will be deserialized and passed as "
            "**kwargs to the model constructor",
        )
        self.parser.add_argument(
            "--dataset-name",
            required=False,
            help="The name of the Huggingface `datasets` library dataset to predict on",
        )
        self.parser.add_argument(
            "--split",
            required=False,
            help="The split of the Huggingface `datasets` dataset to use",
        )
        self.parser.add_argument(
            "--input-files",
            required=False,
            nargs="+",
            help="The input file(s) to pass to the dataset reader",
        )
        self.parser.add_argument(
            "--dataset-reader",
            required=False,
            help="The name of the dataset reader to use",
        )
        self.parser.add_argument(
            "--dataset-reader-args",
            required=False,
            help="A serialized json object which will be deserialized and passed as "
            "**kwargs to the dataset reader constructor",
        )
        self.parser.add_argument(
            "--output-file",
            required=True,
            help="The name of the output jsonl file that the model predictions will be written to",
        )
        self.parser.add_argument(
            "--output-writer",
            required=False,
            default="default",
            help="The name of the class to use to write the predictions to the output file",
        )
        self.parser.add_argument(
            "--output-writer-args",
            required=False,
            help="A serialized json object which will be deserialized and passed as "
            "**kwargs to the output writer constructor",
        )
        self.parser.add_argument(
            "--log-file",
            required=False,
            help="The file where the log should be written",
        )
        self.parser.add_argument(
            "--silent",
            required=False,
            action="store_true",
            help="Indicates the log should not be written to stdout",
        )
        self.parser.set_defaults(func=self.run)

    @staticmethod
    def _check_args(args):
        # --dataset-name and --input-files are exclusive-or
        if (args.dataset_name is not None) is (args.input_files is not None):
            raise ValueError(
                "Exactly one of --dataset-name or --input-files must be set"
            )

        # Both --dataset-name and --split must either be set or not set
        if (args.dataset_name is not None) is not (args.split is not None):
            raise ValueError(
                "Parameters --dataset-name and --split must either both be set "
                "or neither be set"
            )

        # Both --input-files and --dataset-reader must either be set or not set
        if (args.input_files is not None) is not (args.dataset_reader is not None):
            raise ValueError(
                "Parameters --input-files and --dataset-reader must either both be set "
                "or neither be set"
            )

        # If --dataset-reader-args is passed, --dataset-reader must also be used
        if args.dataset_reader_args is not None:
            if args.dataset_reader is None:
                raise ValueError(
                    "Parameter --dataset-reader must be used if --dataset-reader-args "
                    "is also used"
                )

    @overrides
    def run(self, args):
        self._check_args(args)
        prepare_global_logging(args.log_file, args.silent)

        model = load_model(args.model_name, args.model_args)

        if args.dataset_name is not None:
            dataset_reader = HuggingfaceDatasetsDatasetReader(
                args.dataset_name, args.split
            )
            instances = dataset_reader.read()
        else:
            dataset_reader = load_dataset_reader(
                args.dataset_reader, args.dataset_reader_args
            )
            instances = dataset_reader.read(*args.input_files)

        predictions = predict_with_model(model, instances)

        output_writer = load_output_writer(args.output_writer, args.output_writer_args)
        output_writer.write(
            instances, predictions, args.output_file, model_name=args.model_name
        )



from typing import Any, List

from repro.common import Registrable
from repro.data.types import InstanceDict


class OutputWriter(Registrable):
    def write(
        self,
        instances: List[InstanceDict],
        predictions: List[Any],
        output_file: str,
        *args,
        **kwargs
    ) -> None:
        """
        Writes the results of the prediction to the `output_file`. The `instances` and `predictions`
        are parallel lists such that `predictions[i]` is the prediction for `instances[i]`.

        Parameters
        ----------
        instances : List[InstanceDict]
            The instances that were used to get the predictions
        predictions : List[Any]
            The predictions to save
        output_file : str
            The output file
        """
        raise NotImplementedError



import json
import os
from overrides import overrides
from typing import Any, List

from repro.data.output_writers import OutputWriter
from repro.data.types import InstanceDict


@OutputWriter.register("default")
class DefaultOutputWriter(OutputWriter):
    """
    Writes a jsonl file with keys for the `instance_id`, `model_id`, and `prediction`.
    """

    @overrides
    def write(
        self,
        instances: List[InstanceDict],
        predictions: List[Any],
        output_file: str,
        model_name: str,
        *args,
        **kwargs
    ) -> None:
        dirname = os.path.dirname(output_file)
        if dirname:
            os.makedirs(dirname, exist_ok=True)

        with open(output_file, "w") as out:
            for instance, prediction in zip(instances, predictions):
                out.write(
                    json.dumps(
                        {
                            "instance_id": instance["instance_id"],
                            "model_id": model_name,
                            "prediction": prediction,
                        }
                    )
                    + "\n"
                )



import argparse
import json
import spacy


def main(args):
    nlp = spacy.load("en_core_web_sm")
    with open(args.output_file, "w") as out:
        with open(args.input_file, "r") as f:
            for line in f:
                data = json.loads(line)
                summary = data["summary"]["text"]
                sentences = [str(sentence) for sentence in nlp(summary).sents]
                data["summary"]["text"] = sentences
                out.write(json.dumps(data) + "\n")


if __name__ == "__main__":
    argp = argparse.ArgumentParser()
    argp.add_argument("--input-file", required=True)
    argp.add_argument("--output-file", required=True)
    args = argp.parse_args()
    main(args)



"""
Tokenizes the reference summaries using the instructions from Liu & Lapata (2019)
"""
import argparse
import json
import os

from repro.common import TemporaryDirectory
from repro.common.docker import make_volume_map, run_command
from repro.common.io import write_to_text_file, read_jsonl_file
from repro.common.util import flatten


def main(args):
    instances = read_jsonl_file(args.input_jsonl)
    references = [flatten(instance["reference"]["text"]) for instance in instances]

    with TemporaryDirectory() as temp:
        host_input_dir = f"{temp}/input"
        host_output_dir = f"{temp}/output"
        volume_map = make_volume_map(host_input_dir, host_output_dir)
        container_input_dir = volume_map[host_input_dir]
        container_output_dir = volume_map[host_output_dir]

        host_input_file = f"{host_input_dir}/references.txt"
        container_input_file = f"{container_input_dir}/references.txt"

        write_to_text_file(references, host_input_file)

        # Run inference. The output_dir must exist before running
        # the docker command
        os.makedirs(host_output_dir)
        host_output_file = f"{host_output_dir}/references.txt"
        container_output_file = f"{container_output_dir}/references.txt"
        command = (
            f"python preprocess.py"
            f"  --input-file {container_input_file}"
            f"  --output-file {container_output_file}"
        )

        run_command(
            "liu2019",
            command,
            volume_map=volume_map,
            network_disabled=True,
        )

        # Load the output summaries
        references = []
        with open(host_output_file, "r") as f:
            for line in f:
                sentences = line.strip().split(" [CLS] [SEP] ")
                references.append(sentences)

    with open(args.output_jsonl, "w") as out:
        for instance, reference in zip(instances, references):
            instance["reference"] = {"text": reference}
            out.write(json.dumps(instance) + "\n")


if __name__ == "__main__":
    argp = argparse.ArgumentParser()
    argp.add_argument("--input-jsonl", required=True)
    argp.add_argument("--output-jsonl", required=True)
    args = argp.parse_args()
    main(args)



import os
from typing import List, Tuple


def get_testing_device_parameters() -> List[Tuple[int]]:
    """
    Gets the list of device parameters to use for unittesting based
    on the value of the TEST_DEVICES environment variable.
    """
    devices = os.environ.get("TEST_DEVICES", "-1").split(",")
    return [(int(device),) for device in devices]
